//===----------------------------------------------------------------------===//
//
// Philipp Schubert
//
//    Copyright (c) 2017 - 2021
//    Secure Software Engineering Group
//    Heinz Nixdorf Institute
//    Paderborn University
//    philipp.schubert@upb.de
//
//===----------------------------------------------------------------------===//

#include <chrono>
#include <initializer_list>
#include <iostream>
#include <stdexcept>
#include <type_traits>
#include <vector>

template <typename T> class matrix {
public:
  matrix(size_t rows, size_t columns);
  matrix(size_t rows, size_t columns, const T &ival);
  matrix(std::initializer_list<std::initializer_list<T>> imat);
  T &operator()(size_t row, size_t column);
  const T &operator()(size_t row, size_t column) const;
  size_t num_elements() const noexcept;
  size_t num_rows() const noexcept;
  size_t num_columns() const noexcept;
  friend matrix operator*(const matrix &lhs, const T &scale);
  friend matrix operator*(const matrix &lhs, const matrix &rhs);
  friend bool operator==(const matrix &lhs, const matrix &rhs);
  friend bool operator!=(const matrix &lhs, const matrix &rhs);
  friend std::ostream &operator<<(std::ostream &os, const matrix &m);

private:
  size_t rows;
  size_t columns;
  std::vector<T> data;
};

int main() {
  // // TODO comment-in the following code as needed to test your implementation
  // // above.
  // matrix<double> a(3, 3, 3);
  // a = a * 2;
  // matrix<double> b(3, 3, 4);
  // auto start = std::chrono::steady_clock::now();
  // matrix<double> c = a * b;
  // auto end = std::chrono::steady_clock::now();
  // auto duration =
  //     std::chrono::duration_cast<std::chrono::nanoseconds>(end - start).count();
  // std::cout << "multiplied a:\n";
  // std::cout << a << '\n';
  // std::cout << "with b:\n";
  // std::cout << b << '\n';
  // std::cout << "in " << duration << "ns\n";
  // std::cout << "result is:\n";
  // std::cout << c << '\n';

  // matrix<double> d = {{1, 2, 3}, {4, 5, 6}, {7, 8, 9}};
  // matrix<double> e = {{9, 8, 7}, {6, 5, 4}, {3, 2, 1}};
  // start = std::chrono::steady_clock::now();
  // matrix<double> f = d * e;
  // end = std::chrono::steady_clock::now();
  // duration =
  //     std::chrono::duration_cast<std::chrono::nanoseconds>(end - start).count();
  // std::cout << "multiplied d:\n";
  // std::cout << d << '\n';
  // std::cout << "with e:\n";
  // std::cout << e << '\n';
  // std::cout << "in " << duration << "ns\n";
  // std::cout << "result is:\n";
  // std::cout << f << '\n';
  return 0;
}
